#include <string.h>
#include <stdio.h>

#include "ecc.h"


/* some basic bit-manipulation routines that act on these vectors follow */

int bitstr_is_clear(const bitstr_t x)
{
  int i;
  for(i = 0; i < NUMWORDS && ! *x++; i++);
  return i == NUMWORDS;
}

/* return the number of the highest one-bit + 1 */
int bitstr_sizeinbits(const bitstr_t x)
{
  int i;
  uint32_t mask;
  for(x += NUMWORDS, i = 32 * NUMWORDS; i > 0 && ! *--x; i -= 32);
  if (i) {
    for(mask = ((uint32_t) 1) << 31; ! (*x & mask); mask >>= 1, i--);
  }
  return i;
}

/* left-shift by 'count' digits */
void bitstr_lshift(bitstr_t A, const bitstr_t B, int count)
{
  int i, offs = 4 * (count / 32);
  memmove((char*)A + offs, B, sizeof(bitstr_t) - offs);
  memset(A, 0, offs);
  if (count %= 32) {
    for(i = NUMWORDS - 1; i > 0; i--)
      A[i] = (A[i] << count) | (A[i - 1] >> (32 - count));
    A[0] <<= count;
  }
}


/* (raw) import from a byte array */
void bitstr_import(bitstr_t x, const char *s)
{
  int i;
  
  for(x += NUMWORDS, i = 0; i < NUMWORDS; i++, s += 4)
    *--x = CHARS2INT(s);
}

/* (raw) export to a byte array */
void bitstr_export(char *s, const bitstr_t x)
{
  int i;
  for(x += NUMWORDS, i = 0; i < NUMWORDS; i++, s += 4)
    INT2CHARS(s, *--x);
}

/* export as hex string (null-terminated!) */
void bitstr_to_hex(char *s, const bitstr_t x)
{  
  int i;
  for(x += NUMWORDS, i = 0; i < NUMWORDS; i++, s += 8)
    sprintf(s, "%08x", *--x);  // TODO incorrect results - need different implementation
}


uint8_t letter2bin (const char c)
{
  return c>'9' ? c+10-(c>='a'?'a':'A') : 
  c-'0';
}

uint8_t octet2bin(const char* octet)
{
  return (letter2bin(octet[0])<<4) | letter2bin(octet[1]);
}

void bin2letter(char *c, uint8_t b)
{
  *c = b<10? '0'+b : 'A'+b-10;
}

void bin2octet(char *octet, uint8_t bin)
{
  bin2letter(octet,bin>>4);
  bin2letter(octet+1,bin&0x0f);
}


uint32_t getword32(const char *s)
{
  //little endian
  union {
    uint32_t i; 
    uint8_t c[sizeof(uint32_t)];
  } 
  r;
  r .c[3]=octet2bin(s);
  r.c[2]=octet2bin(s+2);
  r.c[1]=octet2bin(s+4);
  r.c[0]=octet2bin(s+6);
  return r.i;
}


int x_strspn(const char *string, const char *charset)
{   
const char *p, *q;  
for (q = string; *q != '\0'; ++q) {
           for (p = charset; *p != '\0' && *p != *q; ++p);
	   if (*p == '\0') break;  
         }
return (q - string);
}



/* import from a hex string */
int bitstr_parse(bitstr_t x, const char *s)
{
  int len;
  if ((s[len = x_strspn(s, "0123456789abcdefABCDEF")]) || (len > NUMWORDS * 8) ) {
    return -1;
  }
  bitstr_clear(x);
  x += len / 8;
  if (len % 8) {
    *x=getword32(s);
    *x >>= 32 - 4 * (len % 8);
    s += len % 8;
    len &= ~7;
  }
  for(; *s; s += 8)
    *--x = getword32(s);
  return len;
}

/******************************************************************************/



