
//#include "Arduino.h"
#define DEGREE 53

#include "ecc.h"

void printP(const char* data){
   while(pgm_read_byte(data) != 0x00) {
       Serial.write(pgm_read_byte(data));
       data++;  
 }
}


void generate_random(char* buffer, int length) {
  for(int i = 0; i<length;i++){
    buffer[i] = 0xff & random();
  }
}

void printCharHex(unsigned char c){
  unsigned char n;
  n = c / 16;
  Serial.write( n < 10 ? n + '0' : n + 'A' - 10);
  n = c % 16;
  Serial.write( n < 10 ? n + '0' : n + 'A' - 10);
}

void printBITSTR(const bitstr_t d, const char* label){
  printP(label);
  char* dd = (char *) d;
  int sz = sizeof (bitstr_t);
   while(sz-- > 0) {
       printCharHex(*dd++);
     }
     printP(PSTR("\n"));
}

void printPHEX(const char* data, int sz){
   while(sz-- > 0) {
       printCharHex(pgm_read_byte(data++));
       Serial.print(" ");
   }
}

void print(const char* str){
  Serial.print(str);
}

void print(int i){
  Serial.print(i);
}


void encryption_decryption_demo(const char *text, const char *public_x,
                                const char *public_y, const char *priv)
{
  int len = strlen(text) + 1;
  char *encrypted = (char*)malloc(len + ECIES_OVERHEAD);
  char *decrypted = (char*)malloc(len);

  printP(PSTR("plain text: "));
  print(text);  
  printP(PSTR("\n"));

  print("Px: ");
  print(public_x);
  print(" Py: ");
  print(public_y);
  print(" Pr: ");
  print(priv);
  print(" \n ");
  
  
  
  ECIES_encryption(encrypted, text, len, public_x, public_y);   /* encryption */
  
  printP(PSTR("ECIES_encryption done\n"));

 print("Px: ");
  print(public_x);
  print(" Py: ");
  print(public_y);
  print(" Pr: ");
  print(priv);
  print(" \n ");
  


  if (ECIES_decryption(decrypted, encrypted, len, priv) < 0) /* decryption */
  {
    printP(PSTR("decryption failed!\n"));
  }
  else
  {
    printP(PSTR("after encryption/decryption: "));
    print(decrypted);
    printP(PSTR("\n"));
  }

  free(encrypted);
  free(decrypted);
}

int8_t endianity(void)
{
  int8_t jj= -1;
  union {
    uint32_t i; 
    char z[sizeof(uint32_t)];
  } 
  u;
  u.i=1;
  jj=0; 
  while(!u.z[jj]) jj++;
  return jj;
}



void setup(){

  Serial.begin(9600);

delay(5000);
/*
  Serial.print( "endianity ");
  Serial.print(endianity()); 
  Serial.println();
*/


  int rc;
  rc = bitstr_parse(poly,         "20000000000047"); 
  rc |= bitstr_parse(coeff_b,     "15");
  rc |= bitstr_parse(base_x,      "1649bdeffa5798");
  rc |= bitstr_parse(base_y,      "ebd6b65ff9cfc");
  rc |= bitstr_parse(base_order,  "1ffffffe699516");
#if 0
  int rc;                //01234567890123456789012345678901234567890
  rc = bitstr_parse(poly, "800000000000000000000000000000000000000c9");
  rc |= bitstr_parse(coeff_b, "20a601907b8c953ca1481eb10512f78744a3205fd");
  rc |= bitstr_parse(base_x, "3f0eba16286a2d57ea0991168d4994637e8343e36");
  rc |= bitstr_parse(base_y, "0d51fbc6c71a0094fa2cdd545b11c5c0c797324f1");
  rc |= bitstr_parse(base_order, "40000000000000000000292fe77e70c12a4234c33");
#endif

  if (rc == -1) 
    print("error parsing ECC parameters\n");

   char test[300];
 
   sprintf(test,"%d",bitstr_sizeinbits(poly)  ); print(test);print("\n");
   sprintf(test,"%d",bitstr_sizeinbits(coeff_b)  ); print(test);print("\n");
   sprintf(test,"%d",bitstr_sizeinbits(base_x)  ); print(test);print("\n");
   sprintf(test,"%d",bitstr_sizeinbits(base_y)  ); print(test);print("\n");
   sprintf(test,"%d",bitstr_sizeinbits(base_order)  ); print(test);print("\n");
  
//  ECIES_generate_key_pair();          /* generate a public/private key pair */

printP(PSTR("******************************************\n"));
  
  encryption_decryption_demo("This secretmsglhkjhlkh\n",
  "eb28797f70e1", 
  "112465e7973e44",
  "15ab45d7bd");

#if 0
 encryption_decryption_demo("This secret demo message will be ECIES encrypted",
                             "1c56d302cf642a8e1ba4b48cc4fbe2845ee32dce7",
                             "45f46eb303edf2e62f74bd68368d979e265ee3c03",
                             "0e10e787036941e6c78daf8a0e8e1dbfac68e26d2");
  
#endif

}



void loop(){

  
}

